//
//  MINTLDropZone.swift
//  TreeOutlinerSwift
//
//  Created by narita on 2019/02/25.
//  Copyright © 2019 narita. All rights reserved.
//

import Cocoa




@objc public class MINTLDropZone: NSBox
{
    // MARK: - class methods
    
    // MARK: - init methods
        
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods

    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods Drag & Drop

    override public func draggingEntered(_ sender: NSDraggingInfo) -> NSDragOperation
    {
        // delegateへデータを渡して可否をもらう。
        if let theDelegate = self.delegate as MINTLDropZoneDelegate?
        {
            if theDelegate.responds(to: #selector(MINTLDropZoneDelegate.dropZone(_: validateDrop:)))
            {
                let theDragOp = theDelegate.dropZone!(self, validateDrop: sender)
                
                self.higlightForDragging = theDragOp != []
                
                return theDragOp
            }
        }
        
        self.higlightForDragging = false

        return []
    }
    
    override public func draggingExited(_ sender: NSDraggingInfo?)
    {
        self.higlightForDragging = false
    }
    
    override public func performDragOperation(_ sender: NSDraggingInfo) -> Bool
    {
        // Delegateへデータを渡して、可否をもらう。
        if let theDelegate = self.delegate as MINTLDropZoneDelegate?
        {
            if theDelegate.responds(to: #selector(MINTLDropZoneDelegate.dropZone(_: acceptDrop:)))
            {
                return theDelegate.dropZone!(self, acceptDrop: sender)
            }
        }

        
        return false
    }
    
    override public func concludeDragOperation(_ sender: NSDraggingInfo?)
    {
        self.higlightForDragging = false
    }
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    override public func draw(_ dirtyRect: NSRect)
    {
        if self.higlightForDragging
        {
            NSGraphicsContext.saveGraphicsState()
            
            NSColor.keyboardFocusIndicatorColor.withAlphaComponent(1.0).set()
            NSFocusRingPlacement.only.set()
            
            NSBezierPath(rect: self.bounds.insetBy(dx: 2, dy: 2)).fill()

            NSGraphicsContext.restoreGraphicsState()
        }
        
        super.draw(dirtyRect)
    }
    
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    var higlightForDragging: Bool = false
    {
        didSet(newValue)
        {
            self.needsDisplay = true
        }
    }
    
    @IBOutlet weak open var delegate: MINTLDropZoneDelegate?

    // MARK: - Utility methods

}

@objc public protocol MINTLDropZoneDelegate : NSObjectProtocol
{
    // ドロップできるか、できるならばどのDragOperationかを返す。できない場合は[]を返す。
    @objc optional func dropZone(_ dropZone: MINTLDropZone, validateDrop info: NSDraggingInfo) -> NSDragOperation
    
    // ドロップされたと時のカスタマイズに使う、Dropを受け入れたならばtrueを返す。
    // delegateでDropされた時の各種処理を行う。
    @objc optional func dropZone(_ dropZone: MINTLDropZone, acceptDrop info: NSDraggingInfo) -> Bool
    
}
